import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { XRUpdateParams } from '../../core/XRClass'
import XRController, { XRControllerConstructor } from '../../core/XRController'
import ControllerType from '../../enums/ControllerType'
import { IFocusable } from './Focusable'
import FocuserKeyboard from './FocuserKeyboard'
import FocuserQuest2 from './FocuserQuest2'

export interface IFocuser extends XRController {
	focusTargets: IFocusable[]
}

export default function Focuser() {
	return [FocuserMixIn]
}

function FocuserMixIn<T extends XRController>(
	Base: XRControllerConstructor<T>
): ConstructorOf<T & IFocuser> {
	let CoreBase: any = FocuserCore(Base)
	switch (Base.controllerType) {
		case ControllerType.KEYBOARD:
			return FocuserKeyboard(CoreBase) as any

		case ControllerType.QUEST2:
			return FocuserQuest2(CoreBase) as any
	}
	return CoreBase
}

function FocuserCore<T extends XRController>(Base: ConstructorOf<T>): ConstructorOf<T & IFocuser> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class FocuserCoreMixIn extends Base {
		focusTargets: IFocusable[]

		defaults() {
			return { ...super.defaults(), focusTargets: [] }
		}

		update(p?: XRUpdateParams) {
			this.focusTargets = this.project.scene.all
				.map(m => m as unknown as IFocusable)
				.filter(m => !!m.isFocusable)
			super.update(p)
		}
	}
	return FocuserCoreMixIn as ConstructorOf<T & IFocuser>
}
