import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { Object3D, Raycaster, Vector2 } from 'three'
import KeyboardController from '../../controllers/KeyboardController'
import { XRUpdateParams } from '../../core/XRClass'
import { IFocuser } from './Focuser'

export default function FocuserKeyboard<T extends KeyboardController>(
	Base: ConstructorOf<T & IFocuser>
): ConstructorOf<T & IFocuser> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class FocuserKeyboardMixIn extends Base {
		focusRaycaster: Raycaster
		focusMousePos: Vector2

		defaults() {
			return {
				...super.defaults(),
				focusRaycaster: new Raycaster(),
				focusMousePos: new Vector2(),
			}
		}

		update(p?: XRUpdateParams) {
			const viewer = this.project.viewer
			const plyr = this.project.player
			if (!viewer) return
			const { w: width, h: height } = viewer.dims
			const offset = this.mousePos.offset
			this.focusMousePos.x = (offset.x / width) * 2 - 1
			this.focusMousePos.y = 0 - ((offset.y / height) * 2 - 1)

			this.focusRaycaster.setFromCamera(this.focusMousePos, plyr.camera)

			const targets = this.focusTargets.map(m => m as Object3D)
			const intersects = this.focusRaycaster.intersectObjects(targets)
			const hitTargetIdx = intersects.length > 0 ? targets.indexOf(intersects[0].object) : -1

			if (hitTargetIdx >= 0) this.focusTargets[hitTargetIdx].focus()
			this.focusTargets.forEach((target, idx) => {
				if (idx !== hitTargetIdx && target.isFocused) target.blur()
			})
			super.update(p)
		}
	}
	return FocuserKeyboardMixIn as any
}
