import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { MeshLambertMaterial, Object3D, Raycaster, Vector3 } from 'three'
import Quest2Controller from '../../controllers/Quest2Controller'
import { XRUpdateParams } from '../../core/XRClass'
import MotionType from '../../enums/MotionType'
import Box from '../../objects/Box'
import { IFocuser } from './Focuser'

export default function FocuserQuest2<T extends Quest2Controller>(
	Base: ConstructorOf<T & IFocuser>
): ConstructorOf<T & IFocuser> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class FocuserQuest2MixIn extends Base {
		focusRaycaster: Raycaster
		dummy: Box

		defaults() {
			return {
				...super.defaults(),
				focusRaycaster: new Raycaster(),
			}
		}

		init() {
			super.init()
			this.dummy = new Box({
				width: 0.1,
				height: 0.1,
				depth: 0.1,
				mass: 0,
				material: new MeshLambertMaterial({
					color: 0x00ff00,
				}),
			})
			this.project.scene.addMass(this.dummy, MotionType.Kinematic)
		}

		update(p?: XRUpdateParams) {
			super.update(p)
			const ply = this.project.player

			const pp = ply.leftHand._hand.getWorldPosition(new Vector3())
			const pd = ply.leftHand.getWorldDirection(new Vector3()).multiplyScalar(-1)
			this.dummy.position.set(pp.x, pp.y, pp.z)
			this.focusRaycaster.set(pp, pd)
			const targets = this.focusTargets.map(m => m as Object3D)
			const intersects = this.focusRaycaster.intersectObjects(targets)
			const hitTargetIdx = intersects.length > 0 ? targets.indexOf(intersects[0].object) : -1

			if (hitTargetIdx >= 0) this.focusTargets[hitTargetIdx].focus()
			this.focusTargets.forEach((target, idx) => {
				if (idx !== hitTargetIdx && target.isFocused) target.blur()
			})

			if (!!p?.frame && ply.leftHand) {
				ply.leftHand.targetRayPose = p.frame.getPose(
					this.left?.targetRaySpace,
					this.refSpace
				)
				ply.leftHand.triggerTouched = !!this.left?.gamepad?.buttons?.[0]?.touched
			}

			if (!!p?.frame && ply.rightHand) {
				ply.rightHand.targetRayPose = p.frame.getPose(
					this.right?.targetRaySpace,
					this.refSpace
				)
				ply.rightHand.triggerTouched = !!this.right?.gamepad?.buttons?.[0]?.touched
			}
		}
	}
	return FocuserQuest2MixIn as any
}
