import ConstructorOf from '@thats/utils/types/ConstructorOf'
import KeyboardController from '../../controllers/KeyboardController'
import { XRUpdateParams } from '../../core/XRClass'
import { ISelector } from '../selector/Selector'
import { IGrabbable } from './Grabbable'
import { IGrabber } from './Grabber'

export default function GrabberKeyboard<T extends KeyboardController & ISelector & IGrabber>(
	Base: ConstructorOf<T>
): ConstructorOf<T> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class GrabberKeyboardMixIn extends Base {
		temp: boolean = false
		update(p?: XRUpdateParams) {
			// If nothing is grabbed, then bubble up - otherwise
			// we want to override the controller updater as we're in charge
			if (!this.grabbed?.length) {
				if (this.temp) {
					this.temp = false
					return
				}
				return super.update(p)
			}

			// Now lets handle the drag
			this.drag(this.mouseDiff.clone().multiplyScalar(0.005))

			// We need to reset the mouse diff here because we're cancelled the bubble
			this.mouseDiff.set(0, 0)
		}

		onMouseDown(e: MouseEvent) {
			super['onMouseDown'](e)
			this.selected
				.map(m => m as unknown as IGrabbable)
				.filter(m => !!m.isGrabbable)
				.forEach(this.grab.bind(this))
		}

		onMouseUp(e: MouseEvent) {
			super['onMouseUp'](e)
			this.temp = true
			this.ungrabAll()
		}
	}
	return GrabberKeyboardMixIn as any
}
