import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { Vector2, Vector3 } from 'three'
import { XRControllerConstructor } from '../../core/XRController'
import GrabActionType from '../../enums/GrabActionType'
import Grabber, { IGrabber } from '../grabber/Grabber'

export interface IMover extends IGrabber {}

export default function Mover() {
	return [...Grabber(), MoverMixIn]
}

function MoverMixIn<T extends IGrabber>(
	Base: XRControllerConstructor<T>
): ConstructorOf<T & IGrabber> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class MoverMixInClass extends Base {
		drag(v: Vector2) {
			super.drag(v)
			if (this.project.state.grabAction !== GrabActionType.MOVE) return
			if (!v.length()) return

			const t = this.project.player.getWorldDirection(new Vector3())
			this.grabbed.forEach(m => {
				const distanceFromPlayer = m.position
					.clone()
					.sub(this.project.player.position)
					.length()
				const moveFactor = distanceFromPlayer / 4.5 // don't know why its 4.5 but that number works nicely
				m.position.y += v.y * moveFactor
				m.position.x += t.z * -v.x * moveFactor
				m.position.z += t.x * v.x * moveFactor
				m.needsUpdate = true
			})
		}
	}
	return MoverMixInClass as any
}
