import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { Vector2 } from 'three'
import { XRControllerConstructor } from '../../core/XRController'
import GrabActionType from '../../enums/GrabActionType'
import Grabber, { IGrabber } from '../grabber/Grabber'

export interface IResizer extends IGrabber {}

export default function Resizer() {
	return [...Grabber(), ResizerMixIn]
}

function ResizerMixIn<T extends IGrabber>(
	Base: XRControllerConstructor<T>
): ConstructorOf<T & IGrabber> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class ResizerMixInClass extends Base {
		drag(v: Vector2) {
			super.drag(v)
			if (this.project.state.grabAction !== GrabActionType.RESIZE) return
			if (!v.length()) return

			const resizeSpeed = 0.5
			this.grabbed.forEach(m => {
				m.scale.multiplyScalar(1 + -v.x * resizeSpeed)
				m.userData.jiggle = true
				m.needsUpdate = true
			})
		}
	}
	return ResizerMixInClass as any
}
