import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { Vector2, Vector3 } from 'three'
import { XRControllerConstructor } from '../../core/XRController'
import GrabActionType from '../../enums/GrabActionType'
import Grabber, { IGrabber } from '../grabber/Grabber'

export interface IRotator extends IGrabber {}

export default function Rotator() {
	return [...Grabber(), RotatorMixIn]
}

function RotatorMixIn<T extends IGrabber>(
	Base: XRControllerConstructor<T>
): ConstructorOf<T & IGrabber> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class RotatorMixInClass extends Base {
		drag(v: Vector2) {
			super.drag(v)
			if (this.project.state.grabAction !== GrabActionType.ROTATE) return
			if (!v.length()) return

			const rotationSpeed = 1.2
			this.grabbed.forEach(m => {
				m.rotateOnWorldAxis(new Vector3(0, 1, 0), -v.x * rotationSpeed)
				m.rotateOnWorldAxis(new Vector3(1, 0, 0), -v.y * rotationSpeed)
				m.needsUpdate = true
			})
		}
	}
	return RotatorMixInClass as any
}
