import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { XRUpdateParams } from '../../core/XRClass'
import XRController, { XRControllerConstructor } from '../../core/XRController'
import ControllerType from '../../enums/ControllerType'
import Focuser, { IFocuser } from '../focuser/Focuser'
import { ISelectable } from './Selectable'
import SelectorKeyboard from './SelectorKeyboard'
import SelectorQuest2 from './SelectorQuest2'

export interface ISelector extends XRController {
	selected: ISelectable[]
	selectorTargets: ISelectable[]

	select(v: ISelectable): void
	deselect(v: ISelectable): void
	deselectAll(): void
	onChangeSelected(v: ISelectable[]): void
}

export default function Selector() {
	return [...Focuser(), SelectorMixIn]
}

function SelectorMixIn<T extends IFocuser>(
	Base: XRControllerConstructor<T>
): ConstructorOf<T & ISelector> {
	let CoreBase: any = SelectorCore(Base)
	switch (Base.controllerType) {
		case ControllerType.KEYBOARD:
			return SelectorKeyboard(CoreBase) as any

		case ControllerType.QUEST2:
			return SelectorQuest2(CoreBase) as any
	}
	return CoreBase
}

function SelectorCore<T extends IFocuser>(Base: ConstructorOf<T>): ConstructorOf<T & ISelector> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class SelectorCoreMixIn extends Base {
		selectorTargets: ISelectable[]
		selected: ISelectable[]

		defaults() {
			return { ...super.defaults(), selected: [], selectorTargets: [] }
		}

		update(p?: XRUpdateParams) {
			this.selectorTargets = this.project.scene.all
				.map(m => m as unknown as ISelectable)
				.filter(m => !!m.isSelectable)
			super.update(p)
		}

		select(v: ISelectable) {
			this.selected.push(v)
			v.select()
			this.onChangeSelected(this.selected)
		}

		deselect(v: ISelectable) {
			this.selected = this.selected.filter(s => s !== v)
			v.deselect()
			this.onChangeSelected(this.selected)
		}

		deselectAll() {
			this.selected.forEach(i => i.deselect())
			this.selected = []
			this.onChangeSelected(this.selected)
		}

		onChangeSelected(v: ISelectable[]) {}
	}
	return SelectorCoreMixIn as unknown as ConstructorOf<T & ISelector>
}
