import ConstructorOf from '@thats/utils/types/ConstructorOf'
import KeyboardController from '../../controllers/KeyboardController'
import { XRUpdateParams } from '../../core/XRClass'
import KeyCode from '../../enums/KeyCode'
import { IWalker } from './Walker'

export default function WalkerKeyboard<T extends KeyboardController>(
	Base: ConstructorOf<T & IWalker>
): ConstructorOf<T & IWalker> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class WalkerKeyboardMixIn extends Base {
		update(p?: XRUpdateParams) {
			const maxImpulse = this.shiftDown ? 3 : 1
			const maxRoll = 1
			const impulseAcc = 0.1
			const rotateAcc = 0.1
			const impulse = this.impulsion
			const rotation = this.rotation
			const mouseDiff = this.mouseDiff

			// Impulse Forwards
			if (this.isKeyDown(KeyCode.W)) impulse.z = Math.max(-maxImpulse, impulse.z - impulseAcc)
			else if (impulse.z < 0) impulse.z = Math.min(0, impulse.z + impulseAcc)

			// Impulse Backwards
			if (this.isKeyDown(KeyCode.S)) impulse.z = Math.min(maxImpulse, impulse.z + impulseAcc)
			else if (impulse.z > 0) impulse.z = Math.max(0, impulse.z - impulseAcc)

			// Impulse Left
			if (this.isKeyDown(KeyCode.A)) impulse.x = Math.max(-maxImpulse, impulse.x - impulseAcc)
			else if (impulse.x < 0) impulse.x = Math.min(0, impulse.x + impulseAcc)

			// Impulse Right
			if (this.isKeyDown(KeyCode.D)) impulse.x = Math.min(maxImpulse, impulse.x + impulseAcc)
			else if (impulse.x > 0) impulse.x = Math.max(0, impulse.x - impulseAcc)

			// Roll Left
			if (this.isKeyDown(KeyCode.Q)) rotation.z = Math.max(-maxRoll, rotation.z - rotateAcc)
			else if (rotation.z < 0) rotation.z = Math.max(0, rotation.z + rotateAcc)

			// Roll Right
			if (this.isKeyDown(KeyCode.E)) rotation.z = Math.min(maxRoll, rotation.z + rotateAcc)
			else if (rotation.z > 0) rotation.z = Math.max(0, rotation.z - rotateAcc)

			// Yaw
			rotation.y = this.mouseDown ? mouseDiff.x * rotateAcc : 0

			// Pitch
			rotation.x = this.mouseDown ? mouseDiff.y * rotateAcc : 0

			super.update(p)
		}
	}
	return WalkerKeyboardMixIn as any
}
