import ConstructorOf from '@thats/utils/types/ConstructorOf'
import Quest2Controller from '../../controllers/Quest2Controller'
import { XRUpdateParams } from '../../core/XRClass'
import { IWalker } from './Walker'

export default function WalkerQuest2<T extends Quest2Controller>(
	Base: ConstructorOf<T & IWalker>
): ConstructorOf<T & IWalker> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class WalkerQuest2MixIn extends Base {
		defaults() {
			return { ...super.defaults() }
		}

		update(p?: XRUpdateParams) {
			const { z: impulseZ } = this.impulsion
			const { y: rotationY } = this.rotation

			const axesThreshold = 0.5
			const maxImpulse = 1

			const fbValue = this.left.gamepad?.axes[3]
			const lrValue = this.right.gamepad?.axes[2]
			const isFwd = fbValue < 0 - axesThreshold
			const isBwd = fbValue > axesThreshold
			const isLeft = lrValue < 0 - axesThreshold
			const isRight = lrValue > axesThreshold

			// Impulse Forwards
			if (isFwd) this.impulsion.z = Math.max(-maxImpulse, impulseZ + fbValue)
			else if (this.impulsion.z < 0) this.impulsion.z = Math.min(0, impulseZ)

			// Impulse Backwards
			if (isBwd) this.impulsion.z = Math.min(maxImpulse, impulseZ + fbValue)
			else if (this.impulsion.z > 0) this.impulsion.z = Math.max(0, impulseZ)

			// Rotation Left
			if (isLeft) this.rotation.y = Math.min(maxImpulse, rotationY - lrValue)
			else if (this.rotation.y > 0) this.rotation.y = Math.min(0, rotationY)

			// Rotation Right
			if (isRight) this.rotation.y = Math.max(-maxImpulse, rotationY - lrValue)
			else if (this.rotation.y < 0) this.rotation.y = Math.max(0, rotationY)

			super.update(p)
		}
	}
	return WalkerQuest2MixIn as any
}
