import { XRInputSource, XRReferenceSpace } from 'three'
import { XRUpdateParams } from '../core/XRClass'
import XRController from '../core/XRController'
import ControllerType from '../enums/ControllerType'

export default class Quest2Controller extends XRController {
	static controllerType = ControllerType.QUEST2
	refSpace: XRReferenceSpace
	left: XRInputSource
	right: XRInputSource

	defaults() {
		return { ...super.defaults(), controllerType: ControllerType.QUEST2, speed: 0.5 }
	}

	init() {
		super.init()
		this._onInputSourcesChange = this.onInputSourcesChange.bind(this)
		const s = this.project.session
		s.addEventListener('inputsourceschange', this._onInputSourcesChange)
		s.requestReferenceSpace('viewer').then(a => {
			this.refSpace = a
		})
	}

	update(p?: XRUpdateParams) {
		super.update(p)
		const plyr = this.project.player

		if (plyr.leftHand && this.left?.targetRaySpace && this.refSpace && !!p.frame)
			plyr.leftHand.pose = p.frame.getPose(this.left?.targetRaySpace, this.refSpace)

		if (plyr.rightHand && this.right?.targetRaySpace && this.refSpace && !!p.frame)
			plyr.rightHand.pose = p.frame.getPose(this.right?.targetRaySpace, this.refSpace)
	}

	_onInputSourcesChange: any
	onInputSourcesChange() {
		const s = this.project.session
		const p = this.project.player

		this.left = undefined
		this.right = undefined
		s?.inputSources?.forEach(i => {
			if (i.handedness === 'left') this.left = i
			if (i.handedness === 'right') this.right = i
		})

		if (this.left) p.leftHand.visible = true
		if (this.right) p.rightHand.visible = true
	}

	destroy() {
		const s = this.project.session
		const p = this.project.player
		s.removeEventListener('inputsourceschange', this._onInputSourcesChange)
		p.leftHand.visible = false
		p.rightHand.visible = false
	}
}
