"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SkyDome = void 0;
const three_1 = require("three");
const Sphere_1 = __importDefault(require("./Sphere"));
class SkyDome extends Sphere_1.default {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { radius: 500, widthSegments: 32, heightSegments: 15, topColor: new three_1.Color(0x0077ff), bottomColor: new three_1.Color(0xffffff) });
    }
    init() {
        super.init();
        const uniforms = {
            topColor: { value: this.topColor },
            bottomColor: { value: this.bottomColor },
            offset: { value: 0 },
            exponent: { value: 0.6 },
        };
        const vertexShader = `
			varying vec3 vWorldPosition;
			void main() {
				vec4 worldPosition = modelMatrix * vec4( position, 1.0 );
				vWorldPosition = worldPosition.xyz;
				gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );
			}
		`;
        const fragmentShader = `
			uniform vec3 topColor;
			uniform vec3 bottomColor;
			uniform float offset;
			uniform float exponent;
			varying vec3 vWorldPosition;
			void main() {
				float h = normalize( vWorldPosition + offset ).y;
				gl_FragColor = vec4( mix( bottomColor, topColor, max( pow( max( h , 0.0), exponent ), 0.0 ) ), 1.0 );
			}
		`;
        this.material = new three_1.ShaderMaterial({
            uniforms,
            vertexShader,
            fragmentShader,
            side: three_1.BackSide,
        });
        this.materialStack.push(this.material);
    }
}
exports.SkyDome = SkyDome;
//# sourceMappingURL=SkyDome.js.map