import { MathUtils, MeshLambertMaterial, XRPose } from 'three'
import XRObject from '../core/XRObject'
import Cone from '../objects/Cone'
import Cylinder from '../objects/Cylinder'

export default class PlayerHand extends XRObject {
	private _triggerTouched: boolean = false
	_hand: Cylinder
	_ray: Cone

	init() {
		super.init()
		this._hand = new Cylinder({
			radiusTop: 0.01,
			radiusBottom: 0.01,
			height: 0.1,
			material: new MeshLambertMaterial({
				color: 0xffffff,
				transparent: true,
				opacity: 0.1,
			}),
		})
		this._hand.rotateX(MathUtils.degToRad(-90))
		this.add(this._hand)

		this._ray = new Cone({
			radius: 0.003,
			height: 0.4,
			material: new MeshLambertMaterial({
				color: 0xff0000,
				transparent: true,
				opacity: 0.1,
			}),
		})
		this._ray.position.set(0, this._ray.height / 2, 0)
		this.add(this._ray)
	}

	set pose(pose: XRPose) {
		const p = pose?.transform?.position
		const r = pose?.transform?.orientation
		if (p) this.position.set(p.x, p.y, p.z)
		if (r) this.quaternion.set(r.x, r.y, r.z, r.w)
	}

	set targetRayPose(pose: XRPose) {
		const p = pose?.transform?.position
		const r = pose?.transform?.orientation
		if (p) {
			this._ray.position.set(p.x, p.y, p.z - this._ray.height / 2)
			this._ray.position.sub(this.position)
		}
		if (r) {
			this._ray.quaternion.set(r.x, r.y, r.z, r.w)
			this._ray.quaternion.multiply(this.quaternion.clone().inverse())
			this._ray.rotateX(MathUtils.degToRad(-90))
		}
	}

	set triggerTouched(v: boolean) {}
}
