"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Newton = void 0;
const rand_1 = __importDefault(require("@thats/utils/math/rand"));
const three_1 = require("three");
const Walker_1 = __importDefault(require("../behaviours/walker/Walker"));
const Sphere_1 = __importDefault(require("../objects/Sphere"));
const NumberParam_1 = __importDefault(require("../params/NumberParam"));
const BasicScene_1 = __importDefault(require("./BasicScene"));
class Newton extends BasicScene_1.default {
    defaults() {
        return (Object.assign(Object.assign({}, super.defaults()), { key: 'newton', label: 'Newton', params: {
                noOfBalls: new NumberParam_1.default({
                    label: 'No of Balls',
                    defaultValue: 1,
                    min: 1,
                    max: 100,
                    step: 1,
                }),
            }, controllerBehaviours: [(0, Walker_1.default)()] }));
    }
    init() {
        super.init();
        for (let i = 0; i < this.params.noOfBalls.value; i++) {
            const ball = new Sphere_1.default({
                mass: 1,
                radius: 0.4,
                castShadow: true,
                material: new three_1.MeshLambertMaterial({
                    color: Math.random() * 0xffffff,
                }),
            });
            ball.position.set((0, rand_1.default)(-1, 1), 10 + i + Math.random() * 5, (0, rand_1.default)(-1, 1));
            this.addMass(ball);
            ball.setAngularVelocity((0, rand_1.default)(-10, 10), (0, rand_1.default)(-10, 10), (0, rand_1.default)(-10, 10));
        }
    }
}
exports.Newton = Newton;
exports.default = Newton;
//# sourceMappingURL=Newton.js.map