import rand from '@thats/utils/math/rand'
import { MeshLambertMaterial } from 'three'
import Walker from '../behaviours/walker/Walker'
import Sphere from '../objects/Sphere'
import NumberParam from '../params/NumberParam'
import BasicScene from './BasicScene'

export class Newton extends BasicScene {
	defaults() {
		// prettier-ignore
		return ({...super.defaults(),
			key: 'newton',
			label: 'Newton',
			params: {
				noOfBalls: new NumberParam({
					label: 'No of Balls',
					defaultValue: 1,
					min: 1,
					max: 100,
					step: 1,
				}),
			},
			controllerBehaviours: [Walker()],
		})
	}

	init() {
		super.init()

		for (let i = 0; i < this.params.noOfBalls.value; i++) {
			const ball = new Sphere({
				mass: 1,
				radius: 0.4,
				castShadow: true,
				material: new MeshLambertMaterial({
					color: Math.random() * 0xffffff,
				}),
			})
			ball.position.set(rand(-1, 1), 10 + i + Math.random() * 5, rand(-1, 1))
			this.addMass(ball)
			ball.setAngularVelocity(rand(-10, 10), rand(-10, 10), rand(-10, 10))
		}
	}
}
export default Newton
