"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Box_1 = __importDefault(require("../objects/Box"));
const Cone_1 = __importDefault(require("../objects/Cone"));
const Cylinder_1 = __importDefault(require("../objects/Cylinder"));
const Plane_1 = __importDefault(require("../objects/Plane"));
const Sphere_1 = __importDefault(require("../objects/Sphere"));
const Torus_1 = __importDefault(require("../objects/Torus"));
const iterateGeometries_1 = __importDefault(require("./iterateGeometries"));
const createTorusShape_1 = __importDefault(require("./physics/createTorusShape"));
const createTriMeshShape_1 = __importDefault(require("./physics/createTriMeshShape"));
function createCollisionShape(o) {
    const quat = o.quaternion || new three_1.Quaternion(0, 0, 0, 1);
    const geometry = o['geometry'];
    if (o && (geometry === null || geometry === void 0 ? void 0 : geometry.isGeometry)) {
        o.geometry = new three_1.BufferGeometry()['fromGeometry'](geometry);
    }
    const extractData = (o) => {
        const matrixWorld = new three_1.Matrix4().elements;
        const vertices = [];
        const matrices = [];
        const indexes = [];
        (0, iterateGeometries_1.default)(o, {}, (vertexArray, matrixArray, indexArray) => {
            vertices.push(vertexArray);
            matrices.push(matrixArray);
            indexes.push(indexArray);
        });
        return { vertices, matrices, indexes, matrixWorld };
    };
    const d = extractData(o);
    let collisionShape;
    const btHalfExtents = new Ammo.btVector3();
    if (o instanceof Box_1.default) {
        btHalfExtents.setValue(o.width / 2, o.height / 2, o.depth / 2);
        collisionShape = new Ammo.btBoxShape(btHalfExtents);
    }
    else if (o instanceof Sphere_1.default) {
        collisionShape = new Ammo.btSphereShape(o.radius);
    }
    else if (o instanceof Cylinder_1.default) {
        btHalfExtents.setValue(o.radiusTop, o.height / 2, o.radiusBottom);
        collisionShape = new Ammo.btCylinderShape(btHalfExtents);
    }
    else if (o instanceof Cone_1.default) {
        collisionShape = new Ammo.btConeShape(o.radius, o.height);
    }
    else if (o instanceof Torus_1.default) {
        collisionShape = (0, createTorusShape_1.default)(o, quat);
    }
    else if (o instanceof Plane_1.default) {
        collisionShape = (0, createTriMeshShape_1.default)(d.vertices, d.matrices, d.indexes, d.matrixWorld, Object.assign(Object.assign({}, o), { concave: false }));
    }
    return collisionShape;
}
exports.default = createCollisionShape;
//# sourceMappingURL=createCollisionShape.js.map