import ShapeFit from '../../enums/ShapeFit'
import ShapeType from '../../enums/ShapeType'
import computeBounds from '../computeBounds'
import computeHalfExtents from '../computeHalfExtents'
import computeScale from '../computeScale'
import finishCollisionShape from '../finishCollisionShape'
import setDefaultOptions from './setDefaultOptions'

export default function createCylinderShape(vertices, matrices, matrixWorld, options: any = {}) {
	options.type = ShapeType.CYLINDER
	setDefaultOptions(options)

	if (options.fit === ShapeFit.ALL) {
		options.halfExtents = computeHalfExtents(
			computeBounds(vertices, matrices),
			options.minHalfExtent,
			options.maxHalfExtent
		)
	}

	const btHalfExtents = new Ammo.btVector3(
		options.halfExtents.x,
		options.halfExtents.y,
		options.halfExtents.z
	)
	const collisionShape = (() => {
		switch (options.cylinderAxis) {
			case 'y':
				return new Ammo.btCylinderShape(btHalfExtents)
			case 'x':
				return new Ammo.btCylinderShapeX(btHalfExtents)
			case 'z':
				return new Ammo.btCylinderShapeZ(btHalfExtents)
		}
		return null
	})()
	Ammo.destroy(btHalfExtents)

	finishCollisionShape(collisionShape, options, computeScale(matrixWorld, options))
	return collisionShape
}
