import { Matrix4, Vector3 } from 'three'
import ShapeFit from '../../enums/ShapeFit'
import ShapeType from '../../enums/ShapeType'
import computeScale from '../computeScale'
import finishCollisionShape from '../finishCollisionShape'
import setDefaultOptions from './setDefaultOptions'

export const createTriMeshShape = (function () {
	const va = new Vector3()
	const vb = new Vector3()
	const vc = new Vector3()
	const matrix = new Matrix4()
	return function (vertices, matrices, indexes, matrixWorld, options: any = {}) {
		options.type = ShapeType.MESH
		setDefaultOptions(options)

		if (options.fit === ShapeFit.MANUAL) {
			console.warn('cannot use fit: manual with type: mesh')
			return null
		}

		const scale = computeScale(matrixWorld, options)

		const bta = new Ammo.btVector3()
		const btb = new Ammo.btVector3()
		const btc = new Ammo.btVector3()
		const triMesh = new Ammo.btTriangleMesh(true, false)

		for (let i = 0; i < vertices.length; i++) {
			const components = vertices[i]
			const index = indexes[i] ? indexes[i] : null
			matrix.fromArray(matrices[i])
			if (index) {
				for (let j = 0; j < index.length; j += 3) {
					const ai = index[j] * 3
					const bi = index[j + 1] * 3
					const ci = index[j + 2] * 3
					va.set(components[ai], components[ai + 1], components[ai + 2]).applyMatrix4(
						matrix
					)
					vb.set(components[bi], components[bi + 1], components[bi + 2]).applyMatrix4(
						matrix
					)
					vc.set(components[ci], components[ci + 1], components[ci + 2]).applyMatrix4(
						matrix
					)
					bta.setValue(va.x, va.y, va.z)
					btb.setValue(vb.x, vb.y, vb.z)
					btc.setValue(vc.x, vc.y, vc.z)
					triMesh.addTriangle(bta, btb, btc, false)
				}
			} else {
				for (let j = 0; j < components.length; j += 9) {
					va.set(components[j + 0], components[j + 1], components[j + 2]).applyMatrix4(
						matrix
					)
					vb.set(components[j + 3], components[j + 4], components[j + 5]).applyMatrix4(
						matrix
					)
					vc.set(components[j + 6], components[j + 7], components[j + 8]).applyMatrix4(
						matrix
					)
					bta.setValue(va.x, va.y, va.z)
					btb.setValue(vb.x, vb.y, vb.z)
					btc.setValue(vc.x, vc.y, vc.z)
					triMesh.addTriangle(bta, btb, btc, false)
				}
			}
		}

		const localScale = new Ammo.btVector3(scale.x, scale.y, scale.z)
		triMesh.setScaling(localScale)
		Ammo.destroy(localScale)

		// MOD (yandeu): Use btConvexTriangleMeshShape for concave shapes
		let collisionShape
		if (options.concave) collisionShape = new Ammo.btBvhTriangleMeshShape(triMesh, true, true)
		else collisionShape = new Ammo.btConvexTriangleMeshShape(triMesh, true)

		// const collisionShape = new Ammo.btBvhTriangleMeshShape(triMesh, true, true)

		collisionShape.resources = [triMesh]

		Ammo.destroy(bta)
		Ammo.destroy(btb)
		Ammo.destroy(btc)

		finishCollisionShape(collisionShape, options)
		return collisionShape
	}
})()
export default createTriMeshShape
