import useCallforward from '@thatreact/core-hooks/hooks/useCallforward'
import Button from '@thatweb/core-mui/components/Button'
import Col from '@thatweb/core-mui/components/Col'
import SXDiv from '@thatweb/core-mui/components/sx/SXDiv'
import React from 'react'
import XRParam from '../core/XRParam'
import GrabActionType from '../enums/GrabActionType'
import PlayState from '../enums/PlayState'
import ParamBox from './paramboxes/ParamBox'
import { useXRSupported } from './useXRSupported'
import { useXRProjectViewer } from './XRProjectViewerState'

export function XRProjectControls() {
	const { project, setParam, setGrabAction, setGodMode, startXR, pause, play, stop } =
		useXRProjectViewer()
	const xrSupported = useXRSupported()

	const handleChangeGrabAction = useCallforward(
		(v: GrabActionType) => () => setGrabAction(v),
		[setGrabAction]
	)
	const handleFullscreen = React.useCallback(() => alert('to be implemented'), [])
	const handleChangeParam = useCallforward(param => v => setParam(param, v), [setParam])

	const params: XRParam<any>[] = Object.values(project?.scene?.params || {})

	return (
		<SXDiv sx={{ minWidth: 240 }}>
			<Col margin="auto" spacing={2} alignItems="center" sx={{ maxHeight: '100%' }}>
				{/*<Button*/}
				{/*	variant="contained"*/}
				{/*	color="primary"*/}
				{/*	onClick={handleFullscreen}*/}
				{/*	children={'Full Screen'}*/}
				{/*/>*/}
				<Button
					variant="contained"
					color="primary"
					disabled={!xrSupported}
					onClick={startXR}
					children={xrSupported ? 'Enter VR' : 'VR not Available'}
				/>
				<Button
					variant="contained"
					color="primary"
					disabled={project.state.play === PlayState.PLAYING}
					onClick={play}
					children="Play"
				/>
				<Button
					variant="contained"
					color="primary"
					disabled={project.state.play !== PlayState.PLAYING}
					onClick={pause}
					children="Pause"
				/>
				<Button
					variant="contained"
					color="primary"
					disabled={project.state.play === PlayState.STOPPED}
					onClick={stop}
					children="Stop"
				/>
				<Button
					variant="contained"
					color="secondary"
					disabled={project.state.grabAction === GrabActionType.MOVE}
					onClick={handleChangeGrabAction(GrabActionType.MOVE)}
					children="Move"
				/>
				<Button
					variant="contained"
					color="secondary"
					disabled={project.state.grabAction === GrabActionType.ROTATE}
					onClick={handleChangeGrabAction(GrabActionType.ROTATE)}
					children="Rotate"
				/>
				<Button
					variant="contained"
					color="secondary"
					disabled={project.state.grabAction === GrabActionType.RESIZE}
					onClick={handleChangeGrabAction(GrabActionType.RESIZE)}
					children="Resize"
				/>
				<Col
					spacing={2}
					p={2}
					sx={{ flex: 1, overflow: 'scroll', maxHeight: 300, width: '100%' }}
				>
					{params.map((param, idx) => (
						<ParamBox
							key={param.label + idx}
							param={param}
							onChange={handleChangeParam(param)}
						/>
					))}
				</Col>
			</Col>
		</SXDiv>
	)
}
export default XRProjectControls
